from tempfile import NamedTemporaryFile

from bentoml._internal.configuration.containers import BentoMLConfiguration


def get_bentomlconfiguration_from_str(config_str: str):
    tmpfile = NamedTemporaryFile(mode="w+", delete=False)
    tmpfile.write(config_str)
    tmpfile.flush()
    tmpfile.close()

    bentoml_cfg = BentoMLConfiguration(override_config_file=tmpfile.name).as_dict()
    return bentoml_cfg


def test_bentoml_configuration_runner_override():
    OVERRIDE_RUNNERS = """\
runners:
    batching:
        enabled: False
        max_batch_size: 10
    resources:
        cpu: 4
    logging:
        access:
            enabled: False
    test_runner_1:
        resources: system
    test_runner_2:
        resources:
            cpu: 2
    test_runner_gpu:
        resources:
            nvidia.com/gpu: 1
    test_runner_batching:
        batching:
            enabled: True
        logging:
            access:
                enabled: True
"""

    bentoml_cfg = get_bentomlconfiguration_from_str(OVERRIDE_RUNNERS)
    runner_cfg = bentoml_cfg["runners"]

    # test_runner_1
    test_runner_1 = runner_cfg["test_runner_1"]
    assert test_runner_1["batching"]["enabled"] is False
    assert test_runner_1["batching"]["max_batch_size"] == 10
    assert test_runner_1["logging"]["access"]["enabled"] is False
    # assert test_runner_1["resources"]["cpu"] == 4

    # test_runner_2
    test_runner_2 = runner_cfg["test_runner_2"]
    assert test_runner_2["batching"]["enabled"] is False
    assert test_runner_2["batching"]["max_batch_size"] == 10
    assert test_runner_2["logging"]["access"]["enabled"] is False
    assert test_runner_2["resources"]["cpu"] == 2

    # test_runner_gpu
    test_runner_gpu = runner_cfg["test_runner_gpu"]
    assert test_runner_gpu["batching"]["enabled"] is False
    assert test_runner_gpu["batching"]["max_batch_size"] == 10
    assert test_runner_gpu["logging"]["access"]["enabled"] is False
    assert test_runner_gpu["resources"]["cpu"] == 4  # should use global
    assert test_runner_gpu["resources"]["nvidia.com/gpu"] == 1

    # test_runner_batching
    test_runner_batching = runner_cfg["test_runner_batching"]
    assert test_runner_batching["batching"]["enabled"] is True
    assert test_runner_batching["batching"]["max_batch_size"] == 10
    assert test_runner_batching["logging"]["access"]["enabled"] is True
    assert test_runner_batching["resources"]["cpu"] == 4  # should use global


def test_runner_gpu_configuration():
    GPU_INDEX = """\
runners:
    resources:
        nvidia.com/gpu: [1, 2, 4]
"""
    bentoml_cfg = get_bentomlconfiguration_from_str(GPU_INDEX)
    assert bentoml_cfg["runners"]["resources"] == {"nvidia.com/gpu": [1, 2, 4]}

    GPU_INDEX_WITH_STRING = """\
runners:
    resources:
        nvidia.com/gpu: "[1, 2, 4]"
"""
    bentoml_cfg = get_bentomlconfiguration_from_str(GPU_INDEX_WITH_STRING)
    # this behaviour can be confusing
    assert bentoml_cfg["runners"]["resources"] == {"nvidia.com/gpu": "[1, 2, 4]"}


RUNNER_TIMEOUTS = """\
runners:
    timeout: 50
    test_runner_1:
        timeout: 100
    test_runner_2:
        resources: system
"""


def test_runner_timeouts():
    bentoml_cfg = get_bentomlconfiguration_from_str(RUNNER_TIMEOUTS)
    runner_cfg = bentoml_cfg["runners"]
    assert runner_cfg["timeout"] == 50
    assert runner_cfg["test_runner_1"]["timeout"] == 100
    assert runner_cfg["test_runner_2"]["timeout"] == 50
